@@ -50,24 +50,24 @@
 
 #[macro_use]
 extern crate slog;
-extern crate thread_local;
-extern crate take_mut;
 extern crate crossbeam_channel;
+extern crate take_mut;
+extern crate thread_local;
 
 use crossbeam_channel::Sender;
 
-use slog::{Record, RecordStatic, Level, SingleKV, KV, BorrowedKV};
-use slog::{Serializer, OwnedKVList, Key};
+use slog::{BorrowedKV, Level, Record, RecordStatic, SingleKV, KV};
+use slog::{Key, OwnedKVList, Serializer};
 
 use slog::Drain;
-use std::{io, thread};
 use std::error::Error;
 use std::fmt;
 use std::sync;
+use std::{io, thread};
 
-use std::sync::Mutex;
 use std::sync::atomic::AtomicUsize;
 use std::sync::atomic::Ordering;
+use std::sync::Mutex;
 use take_mut::take;
 // }}}
 
@@ -144,6 +144,16 @@ impl Serializer for ToSendSerializer {
         take(&mut self.kv, |kv| Box::new((kv, SingleKV(key, val))));
         Ok(())
     }
+    #[cfg(integer128)]
+    fn emit_u128(&mut self, key: Key, val: u128) -> slog::Result {
+        take(&mut self.kv, |kv| Box::new((kv, SingleKV(key, val))));
+        Ok(())
+    }
+    #[cfg(integer128)]
+    fn emit_i128(&mut self, key: Key, val: i128) -> slog::Result {
+        take(&mut self.kv, |kv| Box::new((kv, SingleKV(key, val))));
+        Ok(())
+    }
     fn emit_usize(&mut self, key: Key, val: usize) -> slog::Result {
         take(&mut self.kv, |kv| Box::new((kv, SingleKV(key, val))));
         Ok(())
@@ -168,7 +178,11 @@ impl Serializer for ToSendSerializer {
     }
 
     #[cfg(feature = "nested-values")]
-    fn emit_serde(&mut self, key: Key, value: &slog::SerdeValue) -> slog::Result {
+    fn emit_serde(
+        &mut self,
+        key: Key,
+        value: &slog::SerdeValue,
+    ) -> slog::Result {
         let val = value.to_sendable();
         take(&mut self.kv, |kv| Box::new((kv, SingleKV(key, val))));
         Ok(())
@@ -195,17 +209,19 @@ impl<T> From<crossbeam_channel::TrySendError<T>> for AsyncError {
 
 impl<T> From<crossbeam_channel::SendError<T>> for AsyncError {
     fn from(_: crossbeam_channel::SendError<T>) -> AsyncError {
-        AsyncError::Fatal(Box::new(
-            io::Error::new(io::ErrorKind::BrokenPipe, "The logger thread terminated"),
-        ))
+        AsyncError::Fatal(Box::new(io::Error::new(
+            io::ErrorKind::BrokenPipe,
+            "The logger thread terminated",
+        )))
     }
 }
 
 impl<T> From<std::sync::PoisonError<T>> for AsyncError {
     fn from(err: std::sync::PoisonError<T>) -> AsyncError {
-        AsyncError::Fatal(Box::new(
-            io::Error::new(io::ErrorKind::BrokenPipe, err.description()),
-        ))
+        AsyncError::Fatal(Box::new(io::Error::new(
+            io::ErrorKind::BrokenPipe,
+            err.description(),
+        )))
     }
 }
 
@@ -268,38 +284,23 @@ where
         self
     }
 
-    fn spawn_thread(
-        self,
-    ) -> (thread::JoinHandle<()>, Sender<AsyncMsg>) {
+    fn spawn_thread(self) -> (thread::JoinHandle<()>, Sender<AsyncMsg>) {
         let (tx, rx) = crossbeam_channel::bounded(self.chan_size);
         let mut builder = thread::Builder::new();
         if let Some(thread_name) = self.thread_name {
             builder = builder.name(thread_name);
         }
         let drain = self.drain;
-        let join = builder.spawn(move || loop {
-            match rx.recv().unwrap() {
-                AsyncMsg::Record(r) => {
-                    let rs = RecordStatic {
-                        location: &*r.location,
-                        level: r.level,
-                        tag: &r.tag,
-                    };
-
-                    drain
-                        .log(
-                            &Record::new(
-                                &rs,
-                                &format_args!("{}", r.msg),
-                                BorrowedKV(&r.kv),
-                            ),
-                            &r.logger_values,
-                        )
-                        .unwrap();
+        let join = builder
+            .spawn(move || loop {
+                match rx.recv().unwrap() {
+                    AsyncMsg::Record(r) => {
+                        r.log_to(&drain).unwrap();
+                    }
+                    AsyncMsg::Finish => return,
                 }
-                AsyncMsg::Finish => return,
-            }
-        }).unwrap();
+            })
+            .unwrap();
 
         (join, tx)
     }
@@ -425,10 +426,11 @@ impl AsyncCore {
         &self,
     ) -> Result<
         &crossbeam_channel::Sender<AsyncMsg>,
-        std::sync::PoisonError<sync::MutexGuard<crossbeam_channel::Sender<AsyncMsg>>>,
+        std::sync::PoisonError<
+            sync::MutexGuard<crossbeam_channel::Sender<AsyncMsg>>,
+        >,
     > {
-        self.tl_sender
-            .get_or_try(|| Ok(self.ref_sender.clone()))
+        self.tl_sender.get_or_try(|| Ok(self.ref_sender.clone()))
     }
 
     /// Send `AsyncRecord` to a worker thread.
@@ -454,25 +456,12 @@ impl Drain for AsyncCore {
         record: &Record,
         logger_values: &OwnedKVList,
     ) -> AsyncResult<()> {
-
-        let mut ser = ToSendSerializer::new();
-        record
-            .kv()
-            .serialize(record, &mut ser)
-            .expect("`ToSendSerializer` can't fail");
-
-        self.send(AsyncRecord {
-            msg: fmt::format(*record.msg()),
-            level: record.level(),
-            location: Box::new(*record.location()),
-            tag: String::from(record.tag()),
-            logger_values: logger_values.clone(),
-            kv: ser.finish(),
-        })
+        self.send(AsyncRecord::from(record, logger_values))
     }
 }
 
-struct AsyncRecord {
+/// Serialized record.
+pub struct AsyncRecord {
     msg: String,
     level: Level,
     location: Box<slog::RecordLocation>,
@@ -481,6 +470,62 @@ struct AsyncRecord {
     kv: Box<dyn KV + Send>,
 }
 
+impl AsyncRecord {
+    /// Serializes a `Record` and an `OwnedKVList`.
+    pub fn from(record: &Record, logger_values: &OwnedKVList) -> Self {
+        let mut ser = ToSendSerializer::new();
+        record
+            .kv()
+            .serialize(record, &mut ser)
+            .expect("`ToSendSerializer` can't fail");
+
+        AsyncRecord {
+            msg: fmt::format(*record.msg()),
+            level: record.level(),
+            location: Box::new(*record.location()),
+            tag: String::from(record.tag()),
+            logger_values: logger_values.clone(),
+            kv: ser.finish(),
+        }
+    }
+
+    /// Writes the record to a `Drain`.
+    pub fn log_to<D: Drain>(self, drain: &D) -> Result<D::Ok, D::Err> {
+        let rs = RecordStatic {
+            location: &*self.location,
+            level: self.level,
+            tag: &self.tag,
+        };
+
+        drain.log(
+            &Record::new(
+                &rs,
+                &format_args!("{}", self.msg),
+                BorrowedKV(&self.kv),
+            ),
+            &self.logger_values,
+        )
+    }
+
+    /// Deconstruct this `AsyncRecord` into a record and `OwnedKVList`.
+    pub fn as_record_values(&self, mut f: impl FnMut(&Record, &OwnedKVList)) {
+        let rs = RecordStatic {
+            location: &*self.location,
+            level: self.level,
+            tag: &self.tag,
+        };
+
+        f(
+            &Record::new(
+                &rs,
+                &format_args!("{}", self.msg),
+                BorrowedKV(&self.kv),
+            ),
+            &self.logger_values,
+        )
+    }
+}
+
 enum AsyncMsg {
     Record(AsyncRecord),
     Finish,
@@ -564,17 +609,22 @@ where
     pub fn chan_size(self, s: usize) -> Self {
         AsyncBuilder {
             core: self.core.chan_size(s),
-            .. self
+            ..self
         }
     }
 
     /// Sets what will happen if the channel is full.
-    pub fn overflow_strategy(self, overflow_strategy: OverflowStrategy) -> Self {
+    pub fn overflow_strategy(
+        self,
+        overflow_strategy: OverflowStrategy,
+    ) -> Self {
         let (block, inc) = match overflow_strategy {
             OverflowStrategy::Block => (true, false),
             OverflowStrategy::Drop => (false, false),
             OverflowStrategy::DropAndReport => (false, true),
-            OverflowStrategy::DoNotMatchAgainstThisAndReadTheDocs => panic!("Invalid variant"),
+            OverflowStrategy::DoNotMatchAgainstThisAndReadTheDocs => {
+                panic!("Invalid variant")
+            }
         };
         AsyncBuilder {
             core: self.core.blocking(block),
@@ -592,7 +642,7 @@ where
     pub fn thread_name(self, name: String) -> Self {
         AsyncBuilder {
             core: self.core.thread_name(name),
-            .. self
+            ..self
         }
     }
 
@@ -717,15 +767,14 @@ impl Drain for Async {
         record: &Record,
         logger_values: &OwnedKVList,
     ) -> AsyncResult<()> {
-
         self.push_dropped(logger_values)?;
 
         match self.core.log(record, logger_values) {
             Ok(()) => {}
             Err(AsyncError::Full) if self.inc_dropped => {
                 self.dropped.fetch_add(1, Ordering::Relaxed);
-            },
-            Err(AsyncError::Full) => {},
+            }
+            Err(AsyncError::Full) => {}
             Err(e) => return Err(e),
         }
 
@@ -741,4 +790,66 @@ impl Drop for Async {
 
 // }}}
 
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    use std::sync::mpsc;
+
+    #[test]
+    fn integration_test() {
+        let (mock_drain, mock_drain_rx) = MockDrain::new();
+        let async_drain = AsyncBuilder::new(mock_drain)
+            .build();
+        let slog = slog::Logger::root(async_drain.fuse(), o!("field1" => "value1"));
+
+        info!(slog, "Message 1"; "field2" => "value2");
+        warn!(slog, "Message 2"; "field3" => "value3");
+        assert_eq!(mock_drain_rx.recv().unwrap(), r#"INFO Message 1: [("field1", "value1"), ("field2", "value2")]"#);
+        assert_eq!(mock_drain_rx.recv().unwrap(), r#"WARN Message 2: [("field1", "value1"), ("field3", "value3")]"#);
+    }
+
+
+    /// Test-helper drain
+    #[derive(Debug)]
+    struct MockDrain {
+        tx: mpsc::Sender<String>,
+    }
+
+    impl MockDrain {
+        fn new() -> (Self, mpsc::Receiver<String>) {
+            let (tx, rx) = mpsc::channel();
+            (Self { tx }, rx)
+        }
+    }
+
+    impl slog::Drain for MockDrain {
+        type Ok = ();
+        type Err = slog::Never;
+
+        fn log(&self, record: &Record, logger_kv: &OwnedKVList) -> Result<Self::Ok, Self::Err> {
+            let mut serializer = MockSerializer::default();
+            logger_kv.serialize(record, &mut serializer).unwrap();
+            record.kv().serialize(record, &mut serializer).unwrap();
+            let level = record.level().as_short_str();
+            let msg = record.msg().to_string();
+            let entry = format!("{} {}: {:?}", level, msg, serializer.kvs);
+            self.tx.send(entry).unwrap();
+            Ok(())
+        }
+    }
+
+    #[derive(Default)]
+    struct MockSerializer {
+        kvs: Vec<(String, String)>,
+    }
+
+    impl slog::Serializer for MockSerializer {
+        fn emit_arguments(&mut self, key: Key, val: &fmt::Arguments) -> Result<(), slog::Error> {
+            self.kvs.push((key.to_string(), val.to_string()));
+            Ok(())
+        }
+    }
+}
+
 // vim: foldmethod=marker foldmarker={{{,}}}
